#
# Copyright (c) 1997-1998, KPMG Peat Marwick LLP, All rights reserved
#
#       Released for public usage, provided:
#               1. KPMG's copyright is acknowledged in this or any derivative work
#               2. Release for direct commercial gain (selling this program
#                  or any derivative work without substantial value added) is
#                  prohibited
#               3. No warantee or support is provided.  If it works for you,
#                  great.  If it fails, s'be'it...
#
#   Bug reports and requests can be directed to BStrauss@kpmg.com
#   

#
# AWK script to update InstallShield .fgl
#
#  This script takes a .dat file, then it expects the file to be a list
#  of the form <fglname> <expression>  And multiple flg files are
#  created...  (If there are multiple expressions, they are processed
#  together into the FGL.
#
#   Invokation of this tool:
#
#       FGLupdte  xx.dat substitution_parm
#
#       %1 in the masks is replaced by substitution_parm
#
# ===================================================================
#
# Data file format is:
#
#  FGLname | Path
#
#  Multiple lines with the same FGLname are merged into one .FGL file
#   Path can be UNC or Directory based
#
#   If PATH is a SINGLE file, that file is put into the FGL
#   If PATH is a DIRECTORY, then the contents of the directory AND
#   all subordinate directories are put into the FGL
#   If PATH has wildcards (? or *), then any file matching the path
#   is put into the FGL as if it was individually named
#
#       CS  | \\PMA0481F26\SYS1\PERFSER2\TEST\INSTALL\NEWSCRIPT\DBMS\
#       ;    Directory ^
#       TMP | D:\TEMP\*.PDF
#       TMP | D:\TEMP\*.AWK
#       ;    WildCards ^
#
#  Lines beginning with ;s are comments       
#
# ===================================================================

#
#   Written for Thompson Automation's TAWK compiler and may not work
#   with other AWKs.
#

function PosLastBackSlash(in_s) {
        local i

        i = length(in_s)
        while ( (i > 0) && (substr(in_s, i, 1) != "\\") ) {
                i--
        }
        return i
}

function PosLastDot(in_s) {
        local i

        i = length(in_s)
        while ( (i > 0) && (substr(in_s, i, 1) != ".") ) {
                i--
        }
        return i
}

function makepath(in_s_pathname,in_s_fn) {
        if (in_s_pathname ~ /[:/\\]$/) {
                return in_s_pathname in_s_fn
        } else {
                return in_s_pathname (OSMODE==4 ? "/" : "\\") in_s_fn
        }
}

function SplitDOSName_Filename(FullName) {

        local i, s_FullName, s_FileName

        s_FullName = FullName

        i = PosLastDot(s_FullName)

        if (i > 0) {
                s_FullName = substr(s_FullName, 1, i-1)
        }

        i = PosLastBackSlash(s_FullName)

        if (i == 0) {
                i = index(s_FileName, ":")
        }

        if (i == 0) {
                s_FileName = s_FullName
        } else {
                s_FileName = substr(s_FullName, i+1)
        }

        return s_FileName
} 

function SplitDOSName_Extension(FullName) {
        local i, s_Ext

        i = PosLastDot(FullName)

        if (i == 0) {
                s_Ext = ""
        } else if (i < 2) {
                s_Ext = ""
        } else {
                s_Ext = substr(FullName, i+1)
        }

        return s_Ext
}

function ProcessEntries(in_s_TopDir, in_s_File, in_s_Mask) {
        local sFileList[], m, i, fn, ii, count, tag, root

#print "\n\nProcessEntries(" in_s_TopDir ", " in_s_File ", " in_s_Mask ")">"Debug.txt"

        m = getdirlist(in_s_Mask, sFileList)

#print "\tgetdirlist returned " m >"Debug.txt"

        for (i in sFileList) {
                
                if (sFileList[i] == ".") { continue }
                if (sFileList[i] == "..") { continue }

                fn = makepath(in_s_Mask, sFileList[i])

                m = substr(filemode(fn), 1, 1)

#print "\tfn=" fn "[" m "]" >"Debug.txt"

                if (m == "d") {
                        ii = index(fn, in_s_TopDir)
                        dir = substr(fn, ii)

                        ii = PosLastBackSlash(dir)
                        if (ii > 0) {
                            tag = substr(dir, 1, ii-1)
                        } else {
                            tag = ""
                        }
                        FGLexpression[in_s_File]["S"][tag][fn] = dir

#print "\tAdded FGLexpression[" in_s_File "][S][" tag "][" fn "] = " dir >"Debug.txt"

                        ProcessEntries(in_s_TopDir, in_s_File, fn)
                }

        }
#print "Return" >"Debug.txt"
}

BEGIN {
         VersionNumber = "1.3ext"
         VersionDate = "25Feb1998"

         print "KPMG Peat Marwick LLP\t\t\t\t\tFGLupdte"
         print "\nVersion " VersionNumber " " VersionDate
         if ( (ARGC < 2) || (ARGC > 3) ) {
             print "**Error, parameters not specified correctly**"
             print "\n\n\tUsage:\n\n\t\tFGLupdte input [substitution_parm]\n"
             exit(1)
         }

        iFileCount = 0
        iFGLCount = 0

        if (ARGC == 3) {
                Substitution_parm = ARGV[2]
                ARGV[2] = ""
        } else {
                Substitution_parm = ""
        }

        fInputFile = ARGV[1]
        fFile = SplitDOSName_Filename(ARGV[1])
        fExt = toupper(SplitDOSName_Extension(ARGV[1]))
        ARGV[1] = ""

        while (getline <fInputFile > 0) {

                if (substr($0, 1, 1) != ";") {

                        i = index($0, "|")

                        if (i > 0) {
                                fgl = substr($0, 1, i-1)
                                fexp = substr($0, i+1)
                                sub(/ +$/, "", fgl)
                                sub(/^ +/, "", fgl)
                                sub(/[\\ ]+$/, "", fexp)
                                sub(/^ +/, "", fexp)

                                # Check for substitution parm...
                                ii = index(fexp, "%1")
                                if (ii > 0) {
                                        fexp = substr(fexp, 1, ii-1) Substitution_parm substr(fexp, ii+2)
                                }

                                FGLname[fgl ".FGL"]++
                                type = "N"
                                value = "Y"
                                if (substr(filemode(fexp), 1, 1) == "d") {
                                        type = "S"
                                        ii = PosLastBackSlash(fexp) + 1
                                        value = substr(fexp, ii)
                                }
                                FGLexpression[fgl ".FGL"][type][""][fexp] = value
                        }
                }

        }

        close(fInputFile)

        for (fFile in FGLname) {
                iFileCount++
        }
        print "\n" iFileCount " FGL file(s) to process...\n" 

#
# When we get here, we have sucked all of the masks for every FGL to
# process into the structure FGLexpression [fglname] [type] [tag] [mask]
#

#
# Process one fgl at a time...
#

        for (fFile in FGLname) {
                print "\nProcessing FGL #" ++iFGLCount ", " fFile "...\n" 

                iItemCount = 0
                iFileCount = 0
                iSubDirCount  = 0

                printf("%-4s %-10s %s\n", "Item", "Type", "Directory/File Mask")
                printf("%-4s %-10s %s\n", "----", "----------", "-------------------")

#
#       We have types N (Normal) and S (Subdirectory)
#               N's become file#= lines under [topdir]
#                               (Note that Ns include wildcard expressions)
#               S's become subdir#= lines under [topdir] and then spawn
#                       entire [xxxx] sections.
#

                print "[TopDir]" >fFile

                for (sMask in FGLexpression[fFile]["N"][""]) {

                        type = "SingleFile"
                        if ( (index(sMask, "?") > 0) ||
                             (index(sMask, "*") > 0) ) {
                                type = "WildCard"
                        }

                        printf("%3d. %-10s %s\n", ++iItemCount, type, sMask)

                        f = findfirst(sMask, 2+4)   # Find Normal, Hidden, System, not Directories!
                        while (f != "") {
                                ii = length(f)
                                if ( (substr(f, ii-1) != "\\.") &&
                                     (substr(f, ii-2) != "\\..") ) {
                                        m = substr(filemode(f), 1, 1)
                                        if (m == "f") {
                                                # Process individual files...
                                                printf("file%d=%s\n", iFileCount++, f) >>fFile
                                        }
                                }
                                f = findnext()
                        }

                }

                # Travel the tree and
                #       1. add all of the subdirectory entries to FGLexpression...
                #       2. create the [topdir] subdir# lines...
                #   Ignore wild card entries...

                for (sMask in FGLexpression[fFile]["S"][""]) {

                        type = "Directory"
                        subdir = FGLexpression[fFile]["S"][""][sMask]
                        printf("%3d. %-10s %s\n", ++iItemCount, type, sMask)
                        printf("subdir%d=%s\n", iSubDirCount++, subdir) >>fFile

                        ProcessEntries(subdir, fFile, sMask)

                }

                #
                # Process each S entry in order... create:
                #       1. []
                #       2. fulldirectory=
                #       3. file#= lines
                #       4. any necessary subdir#= lines...
                #
                for (sTag in FGLexpression[fFile]["S"]) {
                    for (sMask in FGLexpression[fFile]["S"][sTag]) {

                        iFileCount = 0

                        subdir = FGLexpression[fFile]["S"][sTag][sMask]

                        print "\n[" subdir "]" >>fFile
                        print "fulldirectory=" sMask >>fFile
                        f = findfirst(sMask "\\*.*", 2+4)   # Find Normal, Hidden, System, not Directories!
                        while (f != "") {
                                ii = length(f)
                                if ( (substr(f, ii-1) != "\\.") &&
                                     (substr(f, ii-2) != "\\..") ) {
                                        m = substr(filemode(f), 1, 1)
                                        if (m == "f") {
                                                # Process individual files...
                                                printf("file%d=%s\n", iFileCount++, f) >>fFile
                                        }
                                }
                                f = findnext()
                        }

                        iSubDirCount = 0
#print "subdir=" >"Debug.txt"
                        dir = FGLexpression[fFile]["S"][sTag][sMask]
#print "  FGLexpression[" fFile "][S][" sTag "][" sMask "]=" dir >"Debug.txt"
                        for (sMask2 in FGLexpression[fFile]["S"][dir]) {
#print "\t" sMask2 >"Debug.txt"
                                printf("subdir%d=%s\n", iSubDirCount++, FGLexpression[fFile]["S"][dir][sMask2]) >>fFile
                        }

                    }
                }


                print ""                 >>fFile
                print "[General]"        >>fFile
                print "Type=FILELIST"    >>fFile
                print "Version=1.00.000" >>fFile
        
                close(fFile)

        }

#for (i in FGLexpression) {
#  ii = i
#  for (j in FGLexpression[i]) {
#    jj = j
#    for (k in FGLexpression[i][j]) {
#      kk = k
#      for (l in FGLexpression[i][j][k]) {
#        printf("%-20s %-2s %-20s %s[%s]\n", ii, jj, kk, l, FGLexpression[i][j][k][l])>"Debug.txt"
#        kk = ""
#        jj = ""
#        ii = ""
#      }
#    }
#  }
#}

        exit(0)
}

END {
         printf("\n\nDone!\n\n%d FGL file(s) processed\n\n", iFGLCount) 
         print "\nFGLupdte Copyright (c) 1998, KPMG Peat Marwick LLP, All Rights Reserved"
}
